#! perl
# seticcdata.pl: Perl script to parse gmake output for Warpzilla VAC++ 4 build
# (c) Henry Sobotka (sobotka@axess.com) December 1999
# -----------------------------------------------------------------------------------
# For performance-testing
$start = (times)[0];

# Set to 1 for debugging
$chatter = 0;

# XXXX Here until we figure out why a DLL can't be built without an accompanying .exe target
# Set to 1 for exec stub  XXXX SET FROM COMMANDLINE FLAG
$exestub = 1;

# Set to 1 to build static libraries  XXXX SET FROM COMMANDLINE FLAG
$buildstatic = 0;

# File created by running "gmake geticcdata" (rule in OS2.mk)
$FileName = "iccbldata";

# Common .icc header
$Header = "// Generated by seticcdata.pl; edits here will be overwritten.\n\n".
          "if \$__TOS_OS2__ {\n   include \"../config/OS2.icc\"\n}\n\n".
          "option VACDefines,\n       VACOptions";

# Flag for directories containing test programs
$test = 0;

# Current directory
$cdir = undef;

# Target directory
$tdir = undef;

# Root path
$rootpath = undef;

# Library target
$libtarg = undef;

# Array of src file lists
@srcfiles = ();

# Arrays of build targets
@toollist = ();
@dlllist = ();
@testlist = ();
@testdlls = ();
@cpptests = ();

# Array of header paths for TVFS linkage
@tvfslinks = ();

# target:source hashes
%exetargs = ();
%libtargs = ();


# Parse gmake geticcdata output and write .icc files
sub parseiccdata {

    die "Failed to open $FileName\n" if !open(ICC_DATAFILE, "<$FileName");

    while(<ICC_DATAFILE>) {

	chomp;

	# XXXX REDUCE gmake output to required data

	# Current directory recorded by Makefile read
	if (/ICCPROC.+in (.*)/) {

	    print "\n\n", '-' x 20, " IN DIRECTORY $1\n" if ($chatter);
	    $cdir = $1;
	    $test = (/.*\/test/) ? 1 : 0;
	    next;
	}

	# Executable targets
	if (/ICCTRGS = .*\.exe/) {

	    $tdir = $cdir;

	    for $a (split) {

		if ($a =~ /(.+)\.exe/) {

		    my $t = $1;

		    if ($test) {
			if ($t =~ /prstrm/) {push(@cpptests, $t);}
			else {push(@testlist, $t);}
		    }
		    else {push(@toollist, $t)};

		    for $s (@srcfiles) {
			for $f (split(' ', $s)) {
			    push(@{$exetargs{$t}}, $f) if ($f =~ /.+\/$t\./);
			}
		    }
		    
		    die "Failed to open $1.icc\n" if !open(ICC_FILE, ">../$1.icc");
		    print ICC_FILE $Header;

		    # XXXX ADD OUTPUT PATH: [objdir]/dist/[bin|lib] ????
		    print ICC_FILE "\n{\n  target type(exe) \"$t.exe\"\n";

		    print ICC_FILE "  {\n    source \"@{$exetargs{$t}}\"";

		    # XXXX REPLACE with Makefile LIB variables
		    print ICC_FILE "\n    source \"nspr3.lib\", \"plc3.lib\"" if ($test == 1);

		    print ICC_FILE "\n  }\n}";
		    die "Failed to close $1.icc\n" if !close(ICC_FILE);

		    delete $exetargs{$t};
		}
	    }
	    undef @srcfiles;
	    next;
	}

	# Static and DLL targets
	if (/ICCLIBR = .*\/(.+)_s\.lib/) {

	    $tdir = $cdir;
	    $libtarg = $1;
	    if ($test) {push(@testdlls, $libtarg);}
	    else {push(@dlllist, $libtarg)};
	    push(@{$libtargs{$libtarg}}, @srcfiles);
	    undef @srcfiles;
	    next;
	}

	# Source files
	if (/ICCSRCS = (.+)/) {

	    my $src = $1;

	    if ($cdir !~ m/$tdir/ && defined($libtarg)) {
		push(@{$libtargs{$libtarg}}, @srcfiles);
		undef @srcfiles;
                undef $libtarg;
		undef $exetarg;
	    }
	    push(@srcfiles, $src);
	    next;
	}

	# Header directory
	if (/ICCEXPS = .+/) {
	    push(@tvfslinks, $cdir);
	    next;
	}
    }
    die "Failed to close $FileName\n" if !close(ICC_DATAFILE);
    $idx = index($tdir, "nsprpub");
    $rootpath = substr($tdir, 0, $idx);
}


# Write target lists for build loops
# XXXX Roll up into loop
sub writetargdefs {

    $tfile = "../targetlist.icc";
    die "Failed to open $tfile\n" if !open(TARGLIST, ">$tfile");
    print TARGLIST "// Generated by seticcdata.pl; edits here will be overwritten.\n\n";

    $i = 0;
    print TARGLIST "ICCTOOLS = \"";
    for $t (@toollist) {
	next if ($t =~ /nsinstall/);
	if ($i == 0) {print TARGLIST $t; ++$i;}
	else {print TARGLIST "\", \"$t";}
    }
    print TARGLIST "\"";

    $i = 0;
    print TARGLIST "\n\nICCDLLS = \"";
    for $t (@dlllist) {
	if ($i == 0) {print TARGLIST $t; ++$i;}
	else {print TARGLIST "\", \"$t";}
    }
    print TARGLIST "\"";

    if ($buildstatic) {
	$i = 0;
	print TARGLIST "\n\nICCLIBS = \"";
	for $t (@dlllist) {
	    if ($i == 0) {print TARGLIST $t."_s"; ++$i;}
	    else {print TARGLIST "\", \"".$t."_s";}
	}
	print TARGLIST "\"";
    }

    $i = 0;
    print TARGLIST "\n\nICCTESTS = \"";
    for $t (@testlist) {
	if ($i == 0) {print TARGLIST $t; ++$i;}
	else {print TARGLIST "\", \"$t";}
    }
    print TARGLIST "\"";

    $i = 0;
    print TARGLIST "\n\nICCCPPTS = \"";
    for $t (@cpptests) {
	if ($i == 0) {print TARGLIST $t; ++$i;}
	else {print TARGLIST "\", \"$t";}
    }
    print TARGLIST "\"";

    $i = 0;
    print TARGLIST "\n\nICCTDLLS = \"";
    for $t (@testdlls) {
	if ($i == 0) {print TARGLIST $t; ++$i;}
	else {print TARGLIST "\", \"$t";}
    }
    print TARGLIST "\"";

    die "Failed to close $tfile\n" if !close(TARGLIST);
}


sub writedllhash {

    foreach $lib (keys %libtargs) {

	$dfile = "../".$lib.".icc";
	die "Failed to open $tfile\n" if !open(DLLMAKER, ">$dfile");
	print DLLMAKER $Header;
 	print DLLMAKER ",\n       link(exportList, \"".$lib.".exports\")";
	print DLLMAKER "\n{\n";
	print DLLMAKER "  target type(dll) \"".$lib.".dll\"\n  {";
	@srclist = @{$libtargs{$lib}};
	for $s (@srclist) {
	    print DLLMAKER "\n    source ";
	    $i = 0;
	    for $a (split(' ', $s)) {
		if ($i == 0) {print DLLMAKER "\"$a\""; ++$i;}
		else {print DLLMAKER ",\n",' ' x 11,"\"$a\"";}
	    }
	    print DLLMAKER "\n";
	}
	print DLLMAKER "\n    source \"nspr3.lib\"\n" if ($lib !~ /nspr/);
	print DLLMAKER "  }\n";

	if ($exestub && $lib =~ /nspr/) {
	    print DLLMAKER "\n  target type(exe) \"atomic.exe\"\n";
	    print DLLMAKER "  {\n    source \"../pr/tests/atomic.c\"";
	    print DLLMAKER "\n    source \"nspr3.lib\"";
	    print DLLMAKER "\n  }\n";
	}
	print DLLMAKER "}\n";
	die "Failed to close $tfile\n" if !close(DLLMAKER);
    }
}


sub writelibhash {

    foreach $lib (keys %libtargs) {

	$dfile = "../".$lib."_s.icc";
	die "Failed to open $tfile\n" if !open(LIBMAKER, ">$dfile");
	print LIBMAKER $Header;
	print LIBMAKER "{\n";
	print LIBMAKER "  target type(lib) \"".$lib."_s.lib\"\n  {";
	@srclist = @{$libtargs{$lib}};
	for $s (@srclist) {
	    print LIBMAKER "\n    source ";
	    $i = 0;
	    for $a (split(' ', $s)) {
		if ($i == 0) {print LIBMAKER "\"$a\""; ++$i;}
		else {print LIBMAKER ",\n",' ' x 11,"\"$a\"";}
	    }
	    print LIBMAKER "\n";
	}
	print LIBMAKER "  }\n";

	if ($exestub) {
	    print LIBMAKER "\n  target type(exe) \"atomic.exe\"\n";
	    print LIBMAKER "  {\n    source \"../pr/tests/atomic.c\"";
	    print LIBMAKER "\n    source \"nspr3_s.lib\"";
	    print LIBMAKER "\n  }\n";
	}
	print LIBMAKER "}\n";
	die "Failed to close $tfile\n" if !close(LIBMAKER);
    }
}


# Write REXX file for mounting TVFS searchpath drive
sub writetvfsrexx {

    $TVFSfile = "warpztvfs.cmd";
    $mozsrc = $ENV{'MOZ_SRC'};
    $tvfspath = $ENV{'MOZ_TVFS'};
    $tvfsdrive = substr($tvfspath, 0, 2);

    $prolog = "/* REXX script to set up an INCLUDE path for building Mozilla with OS/2. */\n".
              "/* Generated by seticcdata.pl. */\n".
              "/* Edits will be overwritten unless made to the Perl script */\n".
              "/* Adapted from the file created by TVSave */\n\n".
              "/* Load the TVFS REXX interface. */\n".
              "rc = RxFuncAdd('TvLoadFuncs', 'TVRX', 'TvLoadFuncs');\n".
              "if (rc <> 0) then call shutdown;\n".
              "rc = TvLoadFuncs();\n".
              "if (rc <> 0) then call shutdown;\n\n".
              "/* Mount the TVFS drive. */\n".
              "rc = TvMount(\"-c0 ".$tvfsdrive."\");\n".
              "if (rc <> 0) then call shutdown;\n\n".
              "/* Link the Mozilla INCLUDE paths */\n";

    $epilog = "\n/* Unload the TVFS REXX interface. */\n".
              "rc = TvDropFuncs();\n".
              "exit(0);\n\n".
              "shutdown:\n".
              "SAY \"Error setting up TVFS drive.\"\n".
              "call TvUmount(\"*\");\n".
              "call TvDropFuncs;\n".
              "exit(16)\n";

    $catch = "if (rc <> 0) then call shutdown;\n";

    die "Failed to open $TVFSfile\n" if !open(TVFSFILE, ">$TVFSfile");
    print TVFSFILE $prolog;
    for $d (@tvfslinks) {
	$d =~ s/\//\\/g;
	print TVFSFILE "rc = TvLink(\"-drwnt $tvfspath $d\");\n";
	print TVFSFILE $catch;
    }
    # XXXX TEMPORARY LOCATION FOR GENERATED HEADERS
    $ipath = $rootpath."nsprpub/vacpp4/include";
    $ipath =~ s/\//\\/g;
    print TVFSFILE "rc = TvLink(\"-drwnt $tvfspath $ipath\");\n";
    print TVFSFILE $catch;

    print TVFSFILE $epilog;
    die "Failed to close $TVFSfile\n" if !close(TVFSFILE);
}

# For debugging
sub showlibshash {

    print "\n\nFINAL LIBTARGS HASH:\n";
    foreach $lib (keys %libtargs) {
	print "$lib: @{$libtargs{$lib}}\n\n";
    }
}


# Main process
&parseiccdata;
&writetargdefs;
&writedllhash;
&writelibhash if ($buildstatic);
&writetvfsrexx;
&showlibshash if ($chatter);

printf("\nAll done in %.2f sec(s)\n", (times)[0] - $start);
